import 'dart:convert';
import 'dart:io';

import 'package:args/args.dart';
import 'package:path/path.dart' as _path;
import 'package:yaml2podo/resp2yaml.dart';
import 'package:yaml2podo/version.dart';

void main(List<String> args) {
  final argParser = ArgParser();
  argParser.addFlag('help',
      defaultsTo: false, help: 'Displays help information');
  argParser.addOption('out', help: 'Output file name');
  ArgResults argResults;
  void usage() {
    print('Usage: resp2yaml [options] path/to/resposnse.json');
  }

  void error(String message) {
    usage();
    print(message);
    exit(0);
  }

  try {
    argResults = argParser.parse(args);
  } on FormatException {
    usage();
    exit(-1);
  }

  if (argResults['help'] as bool) {
    usage();
    print(argParser.usage);
    exit(0);
  }

  if (argResults.rest.isEmpty) {
    error('Missing json request data file name(s)');
  }

  var outputFullPath = argResults['out'] as String?;
  final fileNames = argResults.rest;
  if (outputFullPath == null) {
    if (fileNames.length > 1) {
      error('Missing output file name');
    }

    final inputFileName = fileNames[0];
    final outputFileName = _path.basenameWithoutExtension(inputFileName);
    final dirName = _path.dirname(inputFileName);
    outputFullPath = _path.join(dirName, outputFileName + '.yaml');
  }

  final lines = <String>[];
  lines.add('# Generated by \'resp2yaml\'');
  lines.add('# Version: ${version}');
  lines.add('# https://pub.dev/packages/yaml2podo');
  lines.add('');
  for (var fileName in fileNames) {
    final inputFile = File(fileName);
    final text = inputFile.readAsStringSync();
    final jsonObject = jsonDecode(text);
    final generator = Resp2YamlGenerator();
    final name = _path.basenameWithoutExtension(fileName);
    lines.add('# ${fileName}');
    final result = generator.generate(jsonObject, [name]);
    lines.addAll(result);
  }

  final outputFile = File(outputFullPath);
  outputFile.writeAsStringSync(lines.join('\n'));
}
