import 'dart:convert';
import 'dart:io';

import 'package:args/args.dart';
// import 'package:dart_style/dart_style.dart';
import 'package:path/path.dart' as _path;
import 'package:yaml/yaml.dart' as _yaml;
import 'package:yaml2podo/version.dart';
import 'package:yaml2podo/yaml2podo_generator.dart';

void main(List<String> args) {
  final argParser = ArgParser();
  argParser.addFlag('camelize',
      defaultsTo: true,
      help:
          'Indicates that property names must be converted to camelCase style');
  argParser.addFlag('format',
      defaultsTo: true,
      help: 'Indicates that the source code should be formatted');
  argParser.addFlag('immutable',
      defaultsTo: true,
      help: 'Indicates that all properties must be declared as immutable');
  argParser.addFlag('store',
      defaultsTo: true,
      help:
          'Indicates that the model prototypes should be stored as comments within the generated code');
  argParser.addFlag('help',
      defaultsTo: false, help: 'Displays help information');
  ArgResults argResults;
  void usage() {
    print('Usage: yaml2podo [options] path/to/json/objects.yaml');
  }

  try {
    argResults = argParser.parse(args);
  } on FormatException {
    usage();
    exit(-1);
  }

  if (argResults['help'] as bool) {
    usage();
    print(argParser.usage);
    exit(0);
  }

  if (argResults.rest.length != 1) {
    usage();
    print('Missing json objects prototype file name');
    exit(0);
  }

  final camelize = argResults['camelize'] as bool;
  final format = argResults['format'] as bool;
  final immutable = argResults['immutable'] as bool;
  final store = argResults['store'] as bool;
  final inputFileName = argResults.rest[0];
  final inputFile = File(inputFileName);
  final data = inputFile.readAsStringSync();
  final source = _yaml.loadYaml(data) ?? <String, dynamic>{};
  if (source is! Map) {
    throw FormatException('Invalid source format');
  }

  final generator =
      Yaml2PodoGenerator(camelize: camelize, immutable: immutable);
  final result = generator.generate(source);
  final dirName = _path.dirname(inputFileName);
  var outputFileName = _path.basenameWithoutExtension(inputFileName);
  outputFileName = _path.join(dirName, outputFileName + '.dart');
  final outputFile = File(outputFileName);
  final lines = <String>[];
  lines.add('// Generated by \'yaml2podo\'');
  lines.add('// Version: ${version}');
  lines.add('// https://pub.dev/packages/yaml2podo');
  lines.add('// ignore_for_file: unused_element');
  lines.add('// @dart = 2.12');
  lines.add('');
  lines.addAll(result);
  if (store) {
    lines.add('');
    lines.add('/*');
    lines.addAll(LineSplitter().convert(data).toList());
    lines.add('*/');
  }

  final code = lines.join('\n');
  if (format) {
    //var formatter = DartFormatter();
    //code = formatter.format(code);
  }

  outputFile.writeAsStringSync(code);
}
