// @dart=2.10

import 'dart:convert';

import 'package:build/build.dart';
//import 'package:dart_style/dart_style.dart';
import 'package:yaml/yaml.dart' as _yaml;
import 'package:yaml2podo/version.dart';
import 'package:yaml2podo/yaml2podo_generator.dart';

Builder yaml2podoBuilder(BuilderOptions options) => Yaml2PodoBuilder();

class Yaml2PodoBuilder implements Builder {
  @override
  final buildExtensions = const {
    '.yaml2podo.yaml': ['.yaml2podo.dart']
  };

  @override
  Future build(BuildStep buildStep) async {
    final info = buildStep.inputId.changeExtension('.dart');
    final inputId = buildStep.inputId;
    final data = await buildStep.readAsString(inputId);
    final yaml = _yaml.loadYaml(data) ?? <String, dynamic>{};
    if (yaml is! Map) {
      throw FormatException('Invalid source format: $inputId');
    }

    // TODO: Add suuport of BuilderOptions
    final camelize = true;
    final format = true;
    final immutable = true;
    final store = true;
    final generator =
        Yaml2PodoGenerator(camelize: camelize, immutable: immutable);
    final result = generator.generate(yaml as Map);
    final lines = <String>[];
    lines.add('// Generated by \'yaml2podo\'');
    lines.add('// Version: ${version}');
    lines.add('// https://pub.dev/packages/yaml2podo');
    lines.add('// ignore_for_file: unused_element');
    lines.add('// @dart = 2.12');
    lines.add('');
    lines.addAll(result);
    if (store) {
      lines.add('');
      lines.add('/*');
      lines.addAll(LineSplitter().convert(data).toList());
      lines.add('*/');
    }

    final code = lines.join('\n');
    if (format) {
      //var formatter = DartFormatter();
      //code = formatter.format(code);
    }

    await buildStep.writeAsString(info, code);
  }
}
